"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_1 = require("@testdeck/mocha");
const chai_1 = require("chai");
const common_errors_1 = require("common-errors");
const src_1 = require("../src");
const Product_1 = require("./classes/Product");
const SkipIndex_1 = require("./classes/SkipIndex");
const factories_1 = require("./factories");
const validate_1 = require("./factories/validate");
class KeyNotInObject {
    constructor() {
        this.deleteThis = true;
    }
}
__decorate([
    src_1.ArrayMember(0),
    __metadata("design:type", Boolean)
], KeyNotInObject.prototype, "deleteThis", void 0);
class UnknownClass {
}
class ChildType {
}
class ParentType {
    constructor() {
        this.child = null;
    }
}
__decorate([
    src_1.ArrayMember(0),
    __metadata("design:type", Object)
], ParentType.prototype, "child", void 0);
let ClassToPlainArrayTest = 
// eslint-disable-next-line @typescript-eslint/no-unused-vars
class ClassToPlainArrayTest {
    async normal() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        const result = src_1.classToPlainArray(Object.assign(new Product_1.Product(), expected, { displayPrice: +expected.displayPrice }));
        validate_1.productArrayValidate(expected, result);
    }
    async array() {
        const testData = factories_1.factory.make(Product_1.Product).many(2);
        const results = src_1.classToPlainArray(testData);
        chai_1.expect(results).property('constructor', Array);
        chai_1.expect(results).length(testData.length);
        for (const [i, expected] of testData.entries()) {
            chai_1.expect(results).property(i.toString());
            validate_1.productArrayValidate(expected, results[i]);
        }
    }
    emptyArray() {
        const result = src_1.classToPlainArray([]);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(0);
    }
    unknownClass() {
        chai_1.expect(() => src_1.classToPlainArray(new UnknownClass())).throw(src_1.UnknownClassError);
    }
    keyNotInObject() {
        const obj = new KeyNotInObject();
        delete obj.deleteThis;
        const result = src_1.classToPlainArray(obj);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(1);
        chai_1.expect(result).property('0').is.undefined;
    }
    missingChildType() {
        const result = src_1.classToPlainArray(new ParentType());
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(1);
        chai_1.expect(result).property('0').is.null;
    }
    undefinedArray() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        const result = src_1.classToPlainArray(Object.assign(expected, { sizes: null }));
        chai_1.expect(result).property('4').is.null;
    }
    arrayIsNotArray() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        chai_1.expect(() => src_1.classToPlainArray(Object.assign(expected, { sizes: 1 }))).throw(common_errors_1.TypeError);
    }
    skipIndex() {
        const expected = factories_1.factory.make(SkipIndex_1.SkipIndex).one();
        validate_1.skipIndexArrayValidate(expected, src_1.classToPlainArray(expected));
    }
};
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ClassToPlainArrayTest.prototype, "normal", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ClassToPlainArrayTest.prototype, "array", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "emptyArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "unknownClass", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "keyNotInObject", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "missingChildType", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "undefinedArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "arrayIsNotArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "skipIndex", null);
ClassToPlainArrayTest = __decorate([
    mocha_1.suite()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
], ClassToPlainArrayTest);
//# sourceMappingURL=classToPlainArray.test.js.map