"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.skipIndexArrayValidate = exports.skipIndexValidate = exports.productArrayValidate = exports.productValidate = exports.defaultValidateOptions = void 0;
const chai_1 = require("chai");
const Product_1 = require("./../classes/Product");
const SkipIndex_1 = require("./../classes/SkipIndex");
exports.defaultValidateOptions = {
    constructor: true,
};
// Product
function productValidate(expected, result, options = exports.defaultValidateOptions) {
    var _a;
    chai_1.expect(result).not.null;
    if (options.constructor) {
        chai_1.expect(result).property('constructor', Product_1.Product);
    }
    chai_1.expect(result).property('id', expected.id);
    chai_1.expect(result).property('price', expected.price);
    // class transform
    chai_1.expect(result).property('displayPrice', expected.displayPrice);
    // nested
    if (expected.color) {
        if (options.constructor) {
            chai_1.expect(result).property('color').property('constructor', Product_1.Color);
        }
        chai_1.expect(result).property('color').property('name', expected.color.name);
    }
    // nested array
    chai_1.expect(result).property('sizes').property('constructor', Array);
    chai_1.expect(result).property('sizes').length(expected.sizes.length);
    for (const [i, { size }] of expected.sizes.entries()) {
        if (options.constructor) {
            chai_1.expect(result).property('sizes').property(i.toString()).property('constructor', Product_1.Size);
        }
        chai_1.expect(result).property('sizes').property(i.toString()).property('size', size);
    }
    // array values
    chai_1.expect(result).property('values').property('constructor', Array);
    chai_1.expect(result).property('values').length(expected.values.length);
    for (const [i, v] of (_a = expected.values.entries()) !== null && _a !== void 0 ? _a : []) {
        chai_1.expect(result).property('values').property(i.toString(), v);
    }
}
exports.productValidate = productValidate;
function productArrayValidate(expected, result) {
    var _a;
    chai_1.expect(result).not.null;
    chai_1.expect(result).property('constructor', Array);
    chai_1.expect(result).property('0', expected.id);
    chai_1.expect(result).property('1', expected.price);
    // class transform
    chai_1.expect(result).property('2', expected.displayPrice);
    // nested
    if (expected.color) {
        chai_1.expect(result).property('3').property('0', expected.color.name);
    }
    else {
        chai_1.expect((result === null || result === void 0 ? void 0 : result[3]) == null).true;
    }
    // nested array
    chai_1.expect(result).property('4').property('constructor', Array);
    chai_1.expect(result).property('4').length(expected.sizes.length);
    for (const [i, { size }] of expected.sizes.entries()) {
        chai_1.expect(result).property('4').property(i.toString()).length(1);
        chai_1.expect(result).property('4').property(i.toString()).property('0', size);
    }
    // array values
    chai_1.expect(result).property('5').property('constructor', Array);
    chai_1.expect(result).property('5').length(expected.values.length);
    for (const [i, v] of (_a = expected.values.entries()) !== null && _a !== void 0 ? _a : []) {
        chai_1.expect(result).property('5').property(i.toString(), v);
    }
}
exports.productArrayValidate = productArrayValidate;
function skipIndexValidate(expected, result, options = exports.defaultValidateOptions) {
    chai_1.expect(result).not.null;
    if (options.constructor) {
        chai_1.expect(result).property('constructor', SkipIndex_1.SkipIndex);
    }
    chai_1.expect(result).property('id', expected.id);
    chai_1.expect(result).property('name', expected.name);
    if (expected.child) {
        if (options.constructor) {
            chai_1.expect(result).property('child').property('constructor', SkipIndex_1.SkipIndexChild);
        }
        chai_1.expect(result).property('child').property('id', expected.child.id);
        chai_1.expect(result).property('child').property('name', expected.child.name);
    }
    else {
        chai_1.expect((result === null || result === void 0 ? void 0 : result.child) == null).true;
    }
    chai_1.expect(result).property('childArray').property('constructor', Array);
    chai_1.expect(result).property('childArray').length(expected.childArray.length);
    for (const [i, childArray] of expected.childArray.entries()) {
        if (options.constructor) {
            chai_1.expect(result).property('childArray').property(i.toString()).property('constructor', SkipIndex_1.SkipIndexChild);
        }
        chai_1.expect(result).property('childArray').property(i.toString()).property('id', childArray.id);
        chai_1.expect(result).property('childArray').property(i.toString()).property('name', childArray.name);
    }
}
exports.skipIndexValidate = skipIndexValidate;
function skipIndexArrayValidate(expected, result) {
    chai_1.expect(result).not.null;
    chai_1.expect(result).property('constructor', Array);
    chai_1.expect(result).property('0', expected.id);
    chai_1.expect(result).property('2', expected.name);
    if (expected.child) {
        chai_1.expect(result).property('4').property('constructor', Array);
        chai_1.expect(result).property('4').property('0', expected.child.id);
        chai_1.expect(result).property('4').property('2', expected.child.name);
    }
    else {
        chai_1.expect((result === null || result === void 0 ? void 0 : result[4]) == null).true;
    }
    chai_1.expect(result).property('6').property('constructor', Array);
    chai_1.expect(result).property('6').length(expected.childArray.length);
    for (const [i, childArray] of expected.childArray.entries()) {
        chai_1.expect(result).property('6').property(i.toString()).property('0', childArray.id);
        chai_1.expect(result).property('6').property(i.toString()).property('2', childArray.name);
    }
}
exports.skipIndexArrayValidate = skipIndexArrayValidate;
//# sourceMappingURL=validate.js.map